-- fifo	upd485506
-- dd modified for registered write enable 
library ieee;
use ieee.std_logic_1164.all;
library exemplar;
use exemplar.exemplar_1164.all;

entity fifo6p is
    generic ( wlength :integer := 5048; width : integer := 8
              );
    port (
-- these activate saveing and loading mem
		save_flag,load_flag : std_logic:='0'; -- 1 for save, 1 for load
			  save_fname,load_fname : string(1 to 12);  -- bypass port file names for save, load
              starta : integer :=0;
              cnt : integer := wlength;

		not_re : in std_logic;
		not_we : in std_logic;
		not_rstr,not_rstw : in std_logic;
		rck,wck : in std_logic;
		md : in std_logic;
		not_oe : in std_logic;
		
		din : in std_logic_vector(width-1 downto 0);
		dout : inout std_logic_vector(width-1 downto 0)	
   );
end fifo6p;

library ieee;
use ieee.std_logic_1164.all;
library exemplar;
use exemplar.exemplar_1164.all;
use std.textio.all;
use work.io_stuff.all;


architecture operate_fifo6p of fifo6p is

type mem_type is array (wlength-1 downto 0) of integer;

shared variable mem : mem_type;


signal inptr : integer :=0;
signal outptr : integer :=0;

signal rmode : std_logic :='0'; -- 0, 16 bit mode, 1- 8 bit mode
signal wmode : std_logic :='0'; -- 0, 16 bit mode, 1- 8 bit mode

signal not_we_reg : std_logic :='0'; -- clocked internal write enable

-- values for -25 parts
constant tAC : time := 18 ns;
constant tDS : time := 7 ns;
constant tDH : time := 3 ns;
constant tHZ : time := 18 ns;
 
FILE load_handle : TEXT;
FILE save_handle : TEXT;


begin




-- write clock setup and hold checks
assert1: for i in 0 to width-1 generate
x : ASSERT not((wck='1' and not wck'stable) and (not din(i)'stable(tDS))) 
        REPORT "setup violation"
        SEVERITY warning;

y : ASSERT not((wck='1' and din(i)'event) and (not wck'stable(tDH))) 
        REPORT "hold violation"
        SEVERITY warning;

end generate;

proc_rmode:
process (md,rck,not_rstr)
begin
  if (rising_edge(rck) and (not_rstr='0')) then
    rmode <= md;
  end if;
end process proc_rmode;

proc_wmode:
process (md,wck,not_rstw)
begin
  if (rising_edge(wck) and (not_rstw='0')) then
    wmode <= md;
  end if;
end process proc_wmode;

proc_input_reg:		  
process (not_rstw,wck,not_we,not_we_reg,inptr,wmode)
begin
  if rising_edge(wck) then
    not_we_reg <= not_we;
      if (not_rstw='0') then
	    inptr <= 0;
      elsif (not_we_reg='0') then
        inptr <= inptr+1;
      end if;
  end if;
end process proc_input_reg;

proc_input:
process (wck,not_we,din,inptr,wmode)
variable temp1,temp2 : std_logic_vector(31 downto 0);
begin
  if ((not_we_reg='0') and rising_edge(wck) and (not_rstw='1')) then
    if (wmode='0') then	-- 16 bit mode
	  mem(inptr) := evec2int(din);
	else -- 8 bit mode
	  if ((inptr rem 2)=0) then
	    temp1(15 downto 8) := int2evec(mem(inptr/2))(15 downto 8);
		temp1(7 downto 0) := din(7 downto 0); 
	    mem(inptr/2) := evec2int(temp1);
	  else
	    temp2(7 downto 0) := int2evec(mem(inptr/2))(7 downto 0);
		temp2(15 downto 8) := din(7 downto 0);
	    mem(inptr/2) := evec2int(temp2);
	  end if;
	end if;
  end if ;
end process proc_input;

proc_output_reg:		  
process (not_rstr,rck,not_re,outptr,rmode)
begin
  if rising_edge(rck) then
    if (not_rstr='0') then
	  outptr <= 0;
    elsif (not_re='0') then
      outptr <= outptr+1;
    end if;
  end if;
end process proc_output_reg;

proc_output:
process (rck,not_re,outptr,rmode,not_oe)
begin
  if rising_edge(rck) then
    if (not_oe='0') then
      if (not_re='0') then
	    if (rmode='0') then
	      dout <= int2evec(mem(outptr))(15 downto 0) after tAC;
	    else
	     if ((outptr rem 2)=0) then
	        dout(7 downto 0) <= int2evec(mem(outptr))(7 downto 0) after tAC;
		  else
	        dout(15 downto 8) <= int2evec(mem(outptr))(15 downto 8) after tAC;
		  end if;
	    end if;
	  end if;
    else
	  for k in dout'reverse_range loop
	    dout(k) <= 'Z' after tHZ;
	  end loop;
	end if;
  end if ;
end process proc_output;

proc_byp_in: process (load_flag)
variable datav,cntr : integer;
variable flag : boolean;
begin
	if (rising_edge(load_flag)) then
	cntr := 0;
    file_open(load_handle,load_fname,READ_MODE);
    for i in starta to wlength-1 loop
	    proc_vram_load(load_handle,datav,flag);
        mem(i) <= int2evec(datav)(width-1 downto 0);
		cntr := cntr+1;
		if (cntr=cnt) then exit; end if;
    end loop;
	file_close(load_handle);
	end if;
end process proc_byp_in;

proc_byp_out: process (save_flag)
variable datav,cntr : integer;
variable flag : boolean;
begin
	if (rising_edge(save_flag)) then
	cntr := 0;
    file_open(save_handle,save_fname,WRITE_MODE);
    for i in starta to wlength-1 loop
        datav := evec2int(mem(i));
	    proc_vram_save(save_handle,datav,width/4,flag);
		cntr := cntr+1;
		if (cntr=cnt) then exit; end if;
    end loop;
	file_close(save_handle);
	end if;
end process proc_byp_out;
	  
	  
  
end operate_fifo6p;
